#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>

#include "xmms-status-plugin.h"
#include "xmms_mini.xpm"

const gchar *plugin_name = "Status Docklet Plugin " VERSION;

static void init(void);
static void cleanup(void);

static gint timeout_callback (gpointer data);
static gint button_press_callback (GtkWidget *widget, GdkEventButton *event);
static gint xmms_remote_function_callback (GtkWidget *widget, gpointer data);

static void toggle_main_window (void);
static void toggle_playlist_window (void);
static void toggle_equalizer_window (void);

static void add_menu_item(GtkWidget *menu, const gchar *name, GtkSignalFunc callback, gpointer data);

static GtkWidget *status_docklet = NULL;
static GtkTooltips *status_tooltips = NULL;
static GtkWidget *status_popup = NULL;

static guint timeout_tag = -1;

GeneralPlugin status_plugin = {
	NULL,				/* handle - filled by xmms */
	NULL,				/* filename - filled by xmms */
	-1,				/* Session */
	"Status Docklet Plugin " VERSION, /* Description */
	init,
	about,
	NULL,				/* Configure */
	cleanup
};

GeneralPlugin *get_gplugin_info(void)
{
	return (&status_plugin);
}

static void init(void)
{
	GdkAtom atom;
	gint data = 1;
	GdkPixmap *icon_pixmap;
	GdkPixmap *icon_mask;
	GtkWidget *icon;

	status_docklet = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(GTK_WIDGET(status_docklet), 22, 22);

	gtk_widget_add_events(GTK_WIDGET(status_docklet), GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(status_docklet), "button_press_event",
			   (GtkSignalFunc)button_press_callback, NULL);

	gtk_widget_realize (GTK_WIDGET(status_docklet));

	icon_pixmap = gdk_pixmap_create_from_xpm_d(status_docklet->window, &icon_mask, 
				   &status_docklet->style->bg[GTK_STATE_NORMAL], 
				   xmms_mini_xpm);

	icon = gtk_pixmap_new ( icon_pixmap, icon_mask );
	gtk_container_add(GTK_CONTAINER(status_docklet), icon);
	gtk_widget_show(icon);
	
	gdk_pixmap_unref(icon_pixmap);
	gdk_bitmap_unref(icon_mask);

	atom = gdk_atom_intern("KWM_DOCKWINDOW", FALSE);

	status_tooltips = gtk_tooltips_new();
	gtk_tooltips_enable(GTK_TOOLTIPS(status_tooltips));

	timeout_tag = gtk_timeout_add(1000, timeout_callback, NULL);

	status_popup = gtk_menu_new();

	add_menu_item(status_popup, _ ("Play"),  (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_play);
	add_menu_item(status_popup, _ ("Pause"), (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_pause);
	add_menu_item(status_popup, _ ("Stop"),  (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_stop);
	add_menu_item(status_popup, _ ("Next"),  (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_playlist_next);
	add_menu_item(status_popup, _ ("Prev"),  (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_playlist_prev);

	add_menu_item(status_popup, NULL, NULL, NULL);

	add_menu_item(status_popup, _ ("Toggle Repeat"),  (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_toggle_repeat);
	add_menu_item(status_popup, _ ("Toggle Shuffle"), (GtkSignalFunc)xmms_remote_function_callback, (gpointer)xmms_remote_toggle_shuffle);
	

	add_menu_item(status_popup, NULL, NULL, NULL);

	add_menu_item(status_popup, _ ("Toggle Main Window"),      (GtkSignalFunc)toggle_main_window, NULL);
	add_menu_item(status_popup, _ ("Toggle Equalizer Window"), (GtkSignalFunc)toggle_equalizer_window, NULL);
	add_menu_item(status_popup, _ ("Toggle Playlist Window"),  (GtkSignalFunc)toggle_playlist_window, NULL);

	add_menu_item(status_popup, NULL, NULL, NULL);

	add_menu_item(status_popup, _ ("About"), (GtkSignalFunc)about, NULL);

	gtk_widget_show(status_docklet);
	gdk_property_change(status_docklet->window, atom, atom, 32, 
			    GDK_PROP_MODE_REPLACE, (guchar *)&data, 1);

}

static void add_menu_item(GtkWidget *menu, const gchar *name, GtkSignalFunc callback, gpointer data)
{
	GtkWidget *item;

	if ( name ) 
		item = gtk_menu_item_new_with_label( name );
	else
		item = gtk_menu_item_new ();

	if ( callback ) 
		gtk_signal_connect (GTK_OBJECT(item), "activate",
				    callback, data);
	gtk_menu_append(GTK_MENU(menu), item);
	gtk_widget_show(item);
}

static void cleanup(void)
{
	if ( timeout_tag != -1 )
		gtk_timeout_remove(timeout_tag);

	gtk_object_destroy(GTK_OBJECT(status_docklet));
	gtk_object_destroy(GTK_OBJECT(status_tooltips));

}


static gint timeout_callback (gpointer data)
{
	static gint last_pos = -1;
	gchar *title;
	gint pos;

	pos = xmms_remote_get_playlist_pos(status_plugin.xmms_session);

	if ( pos != last_pos ) {
		title = xmms_remote_get_playlist_title(status_plugin.xmms_session, pos);
		gtk_tooltips_set_tip(GTK_TOOLTIPS(status_tooltips), GTK_WIDGET(status_docklet),
				     title, NULL);
		
		g_free(title);
		last_pos = pos;
	}

	/* Keep going... */
	return TRUE;
}

static gint button_press_callback (GtkWidget *widget, GdkEventButton *event)
{
	switch ( event->button ) {
	case 1:
		toggle_main_window();
		break;
	case 3:
		gtk_menu_popup (GTK_MENU(status_popup),
				NULL, NULL, NULL, NULL,
				event->button, event->time);
		break;
	default:
		break;
	}


	return TRUE;
}

static gint xmms_remote_function_callback (GtkWidget *widget, gpointer data)
{
	void (*func)(gint xmms_session) = (void(*)(gint xmms_session))data;
	
	func(status_plugin.xmms_session);

	
	return TRUE;
	
}

static void toggle_main_window (void)
{
	gboolean window_shown;;
	
	window_shown = xmms_remote_is_main_win(status_plugin.xmms_session);
	xmms_remote_main_win_toggle(status_plugin.xmms_session, !window_shown);
}

static void toggle_equalizer_window (void)
{
	gboolean window_shown;;
	
	window_shown = xmms_remote_is_eq_win(status_plugin.xmms_session);
	xmms_remote_eq_win_toggle(status_plugin.xmms_session, !window_shown);
}

static void toggle_playlist_window (void)
{
	gboolean window_shown;;
	
	window_shown = xmms_remote_is_pl_win(status_plugin.xmms_session);
	xmms_remote_pl_win_toggle(status_plugin.xmms_session, !window_shown);
}
